#include "anim.h"
#include <gint/display.h>

/* Definition of simple and direction animations. */

#define ANIM1(name) \
    extern anim_frame_t frames_ ## name[]; \
    anim_t anims_ ## name = { 1, { frames_ ## name }};

#define ANIM_2DIRECTIONAL(prefix, suffix) \
    extern anim_frame_t frames_ ## prefix ## _left  ## suffix[]; \
    extern anim_frame_t frames_ ## prefix ## _right ## suffix[]; \
    anim_t anims_ ## prefix ## suffix = { 2, { \
        frames_ ## prefix ## _left  ## suffix, \
        frames_ ## prefix ## _right ## suffix, \
    }}; \

#define ANIM_4DIRECTIONAL(prefix, suffix) \
    extern anim_frame_t frames_ ## prefix ## _up    ## suffix[]; \
    extern anim_frame_t frames_ ## prefix ## _right ## suffix[]; \
    extern anim_frame_t frames_ ## prefix ## _down  ## suffix[]; \
    extern anim_frame_t frames_ ## prefix ## _left  ## suffix[]; \
    anim_t anims_ ## prefix ## suffix = { 4, { \
        frames_ ## prefix ## _up    ## suffix, \
        frames_ ## prefix ## _right ## suffix, \
        frames_ ## prefix ## _down  ## suffix, \
        frames_ ## prefix ## _left  ## suffix, \
    }};

/* Make suffix optional */
#define ANIM2(prefix, ...) \
    ANIM_2DIRECTIONAL(prefix, __VA_OPT__(_ ## __VA_ARGS__))
#define ANIM4(prefix, ...) \
    ANIM_4DIRECTIONAL(prefix, __VA_OPT__(_ ## __VA_ARGS__))

/* Basic skills */
ANIM1(skill_hit);
ANIM1(skill_teleport);
ANIM1(skill_shock);
ANIM1(skill_judgement);
/* Directional skills */
ANIM4(skill_swing);
ANIM4(skill_impale);
ANIM4(skill_bullet);

/* Enemies */
ANIM2(slime, Idle);
ANIM2(slime, Walking);
ANIM2(slime, Hit);
ANIM2(slime, Death);
ANIM2(bat, Idle);
ANIM2(bat, Hit);
ANIM2(bat, Death);
ANIM2(gunslinger, Idle);
ANIM2(gunslinger, Walking);
ANIM2(gunslinger, Reloading);
ANIM2(gunslinger, Fire);
ANIM2(gunslinger, Hit);
ANIM2(gunslinger, Death);

/* Player */
ANIM4(player, Idle);
ANIM4(player, Walking);
ANIM4(player, Attack);
ANIM4(player, Hit);

/* Animation functions. */

fixed_t (anim_duration)(anim_t const *anim)
{
    anim_frame_t const *frame = anim->start[0];
    int ms = 0;
    int i = 0;

    while(frame == anim->start[0] + i) {
        ms += frame->duration;
        frame = frame->next;
        i++;
    }

    return fix(ms) / 1000;
}

bool anim_in(anim_frame_t const *frame, anim_t const *anim, int index)
{
    if(index >= anim->directions)
        return false;

    anim_frame_t *f = anim->start[index];

    for(int i = 0; f == anim->start[index] + i; i++) {
        if(f == frame)
            return true;
        f = f->next;
    }

    return false;
}

void anim_frame_render(int x, int y, anim_frame_t const *frame)
{
    if(!frame) return;
    dsubimage(x - frame->cx, y - frame->cy, frame->sheet,
        frame->x, frame->y, frame->w, frame->h, DIMAGE_NONE);
}

void anim_state_update(anim_state_t *state, fixed_t dt)
{
    if(!state->frame) return;

    state->elapsed += fround(dt * 1000);
    if(state->elapsed < state->frame->duration) return;

    /* Switch to next frame */
    state->elapsed -= state->frame->duration;
    state->frame = state->frame->next;
}
