//---
// fighter: Component for entities that participate in fights
//
// This component gives combat statistics and leveling properties to entities.
//---

#pragma once

#include "comp/entity.h"
#include "fixed.h"

typedef struct
{
    /* Current attack's effect area */
    entity_t *current_attack;
    /* Whether attack follows movement */
    uint8_t attack_follows_movement;
    /* Fighter ID (0 for player) */
    uint8_t identity;
    /* Combat statistics */
    uint16_t HP, ATK, DEF, HP_max;

    /* Number of hits in main attack's combo (TODO: Delegate to weapon?) */
    uint8_t combo_length;
    /* Next hit to be dealt */
    uint8_t combo_next;
    /* Delay until ideal time to start next hit */
    fixed_t combo_delay;
    /* Skill and item cooldowns (F2..F5).The "total" field is updated when
       switching skills/items. The base field is the dynamic value. */
    fixed_t actions_cooldown_total[5];
    fixed_t actions_cooldown[5];

} fighter_t;

/* Damage entity for that amount of raw strength. Returns actual damage after
   DES is subtracted, and randomization. */
int fighter_damage(entity_t *e, int base_damage);
