//---
// enemies: List of enemies and their properties
//---

#pragma once

#include "comp/entity.h"
#include "comp/mechanical.h"
#include "geometry.h"
#include "anim.h"

/* enemy_stat_t: Statistic base and growth */
typedef struct
{
    /* Base value at level 1 */
    fixed_t base;
    /* Initial per-level growth */
    fixed_t growth;
    /* Per-level increase of growth */
    fixed_t affinity;

} enemy_stat_t;

/* enemy_t: Static enemy information */
typedef struct
{
    /* Enemy name (shown in wave information) */
    char const *name;
    /* Map hitbox (represents ground size, collides with walls) */
    rect hitbox;
    /* Sprite hitbox (interacts with attacks and effect areas) */
    rect sprite;
    /* Idle animation, death animation, damage animation */
    anim_t *anim_idle, *anim_walking, *anim_attack, *anim_hit, *anim_death;
    /* Movement parameters */
    mechanical_limits_t limits;
    /* Statistics model */
    enemy_stat_t HP, ATK, DEF;
    /* Shadow size */
    uint8_t shadow_size;
    /* Rendering elevation */
    fixed_t z;

} enemy_t;

/* List of enemies */
enum {
    /* ID 0 is used for the player! */
    ENEMY_SLIME = 1,
    ENEMY_BAT = 2,
    ENEMY_GUNSLINGER = 3,
};

/* Array of enemy data */
extern enemy_t const * const enemies[];

/* Create a new enemy of the given type. */
entity_t *enemy_make(int enemy_id, int level);
